import os
import logging
import subprocess
from typing import Tuple


class ODMProcessMonitor:
    """ODM进程监控器"""

    def __init__(self, mode: str = "快拼模式"):
        self.logger = logging.getLogger('UAV_Preprocess.ODMMonitor')
        self.mode = mode

    def _check_success(self, grid_dir: str) -> bool:
        """检查ODM是否执行成功"""
        success_markers = ['odm_orthophoto', 'odm_georeferencing']
        if self.mode != "快拼模式":
            success_markers.append('odm_texturing')
        return all(os.path.exists(os.path.join(grid_dir, 'project', marker)) for marker in success_markers)

    def run_odm_with_monitor(self, grid_dir: str, grid_idx: int, fast_mode: bool = True) -> Tuple[bool, str]:
        """运行ODM命令"""
        try:
            self.logger.info(f"开始处理网格 {grid_idx + 1}")

            # 构建命令字符串
            command = (
                f"docker run -ti --rm "
                f"-v {grid_dir}:/datasets "
                f"opendronemap/odm "
                f"--project-path /datasets project "
                f"--max-concurrency 10 "
                f"--force-gps "
            )

            if fast_mode:
                command += (
                    f"--feature-quality lowest "
                    f"--orthophoto-resolution 8 "
                    f"--fast-orthophoto "
                    f"--skip-3dmodel "
                )

            command += "--rerun-all"

            # 执行命令
            result = subprocess.run(
                command,
                shell=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL
            )

            # 检查是否成功完成
            if result.returncode == 0 and self._check_success(grid_dir):
                self.logger.info(f"网格 {grid_idx + 1} 处理成功")
                return True, ""

            return False, f"网格 {grid_idx + 1} 处理失败"

        except Exception as e:
            error_msg = f"执行异常: {str(e)}"
            self.logger.error(error_msg)
            return False, error_msg